@@ -12,14 +12,18 @@
 #'   covariate values at which to obtain the fitted predictions.
 #'
 #'   If \code{newdata} is omitted, then the original data used to fit the model
-#'   are used, as extracted by \code{model.frame(object)}.
+#'   are used, as extracted by \code{model.frame(object)}. However this will
+#'   currently not work if the model formula contains functions, e.g.
+#'   \code{~ factor(x)}. The names of the model frame must correspond to
+#'   variables in the original data.
 #'
 #' @param type Character vector for the type of predictions desired.
 #'
-#' * \code{"response"} for mean survival (the default)
+#' * \code{"response"} for mean survival time (the default). \code{"mean"} is
+#'   an acceptable synonym
 #'
-#' * \code{"quantile"} for quantiles of the survival distribution specified by
-#'   \code{p}
+#' * \code{"quantile"} for quantiles of the survival distribution as specified
+#'   by \code{p}
 #'
 #' * \code{"rmst"} for restricted mean survival time
 #'
@@ -113,24 +117,40 @@ predict.flexsurvreg <- function(object,
 {
     if (missing(newdata)) newdata <- model.frame(object)
 
-    assertthat::assert_that(inherits(newdata, "data.frame"),
-                            msg = "`newdata` must inherit class `data.frame`")
-    assertthat::assert_that(is.logical(conf.int), is.logical(se.fit))
-    assertthat::assert_that(all(is.numeric(p), p <= 1, p >=0),
-                            msg = "`p` should be a vector of quantiles between 0 and 1")
-
     assertthat::assert_that(
-        is.numeric(start),
-        msg = "`start` must be a numeric vector of left-truncation times"
+      inherits(newdata, "data.frame"),
+      msg = "`newdata` must inherit the class `data.frame`"
     )
 
-    if (conf.int) assertthat::assert_that(is.numeric(conf.level),
-                                          conf.level > 0, conf.level < 1,
-                                          length(conf.level) == 1,
-                                          msg = "`conf.level` must be length one and between 0 and 1")
+    assertthat::assert_that(
+      is.logical(se.fit),
+      msg = '`se.fit` must be a logical (TRUE/FALSE).'
+    )
 
-    type <- match.arg(type, c("response", "quantile", "link", "lp", "linear",
-                              "survival", "cumhaz", "hazard", "rmst"))
+    assertthat::assert_that(
+      is.logical(conf.int),
+      msg = '`conf.int` must be a logical (TRUE/FALSE).'
+    )
+
+    assertthat::assert_that(
+      all(is.numeric(p), p <= 1, p >= 0),
+      msg = "`p` should be a vector of quantiles in the range [0, 1]"
+    )
+
+    assertthat::assert_that(
+      is.numeric(start),
+      msg = "`start` must be a numeric vector of left-truncation times"
+    )
+
+    if (conf.int) assertthat::assert_that(
+      is.numeric(conf.level),
+      conf.level > 0, conf.level < 1,
+      length(conf.level) == 1,
+      msg = "`conf.level` must be length one and between 0 and 1"
+    )
+
+    type <- match.arg(type, c("mean", "response", "quantile", "link", "lp",
+                              "linear", "survival", "cumhaz", "hazard", "rmst"))
 
     stype <- switch(
         type,
@@ -141,43 +161,84 @@ predict.flexsurvreg <- function(object,
     )
 
     if (stype %in% c("survival", "cumhaz", "hazard")) {
-        if (missing(times)) times <- object$data$Y[, 1][order(object$data$Y[, 1])]
-        assertthat::assert_that(all(is.numeric(times), times > 0),
-                                msg = "`times` must be a vector of positive real-valued numbers.")
+        if (missing(times)) times <- object$data$Y[,1][order(object$data$Y[,1])]
+        assertthat::assert_that(
+          all(is.numeric(times), times >= 0),
+          msg = "`times` must be a vector of non-negative real-valued numbers."
+        )
     } else if (stype == "rmst" && !missing(times)) {
-        assertthat::assert_that(all(is.numeric(times), times > 0),
-                                msg = "`times` must be a vector of positive real-valued numbers.")
+        assertthat::assert_that(
+          all(is.numeric(times), times >= 0),
+          msg = "`times` must be a vector of non-negative real-valued numbers."
+        )
     } else {
         times <- NULL
     }
 
     assertthat::assert_that(
-        length(start) == 1 | length(start) == length(times),
-        msg =
-            paste0(
-                "Length of `start` is ", length(start), ". Length should be 1, or the same length as `times`, which is ", length(times)
-            )
+      length(start) == 1 | length(start) == length(times),
+      msg = paste0(
+        "Length of `start` is ", length(start),
+        ". Length should be 1, or the same length as `times`, which is ",
+        length(times)
+      )
     )
 
     nest_output <- ((stype == "quantile" && length(p) > 1) |
-                        (stype %in% c("survival", "cumhaz", "hazard", "rmst") &&
-                        length(times) > 1))
+                      (stype %in% c("survival", "cumhaz", "hazard", "rmst") &&
+                         length(times) > 1))
 
-    res <- summary(object = object, newdata = newdata, type = stype,
-                   quantiles = p, t = times, start = start,
-                   ci = conf.int, cl = conf.level, se = se.fit,
-                   tidy = FALSE)
+    res <- if (stype %in% c("survival", "hazard", "cumhaz", "rmst")) {
+      summary(
+        object = object,
+        newdata = newdata,
+        type = stype,
+        quantiles = p,
+        t = times,
+        start = start,
+        ci = conf.int,
+        cl = conf.level,
+        se = se.fit,
+        tidy = TRUE,
+        na.action = na.pass,
+        cross = TRUE
+      )
+    } else {
+      # Avoid passing `t = times` for non-time based predictions
+      # to avoid noisy warnings
+      summary(
+        object = object,
+        newdata = newdata,
+        type = stype,
+        quantiles = p,
+        start = start,
+        ci = conf.int,
+        cl = conf.level,
+        se = se.fit,
+        tidy = TRUE,
+        na.action = na.pass,
+        cross = TRUE
+      )
+    }
 
-    res <- rename_tidy(unname(res))
+    res <- tidy_rename(res, stype)
 
-    res <- tibble::tibble(.pred = res)
-
-    if (!nest_output) {
-        res <- tidyr::unnest(res, .pred)
+    if (nest_output) {
+      if (stype == 'quantile') {
+        num_reps <- length(p)
+      } else {
+        num_reps <- length(times)
+      }
+      orig_nrow <- nrow(newdata)
+      res <- dplyr::mutate(res, .id = rep(1:orig_nrow, each = num_reps))
+      res <- tidyr::nest(res, .pred = -.id)
+      res <- dplyr::select(res, .pred)
     }
     res
 }
 
+utils::globalVariables('.id')
+
 tidy_names <- function() {
     tibble::tibble(
         old = c("time", "quantile", "est", "se", "lcl", "ucl"),
@@ -186,11 +247,13 @@ tidy_names <- function() {
     )
 }
 
-rename_tidy <- function(x) {
-    lapply(x, function(x) {
-        for (i in seq_along(tidy_names()$old)) {
-            colnames(x)[colnames(x) == tidy_names()$old[i]] <- tidy_names()$new[i]
-        }
-        x
-    })
+tidy_rename <- function(x, type) {
+  names_tbl <- tidy_names()
+  names_to_change <- names_tbl[names_tbl$old %in% names(x), ]
+  out <- dplyr::select(x, names_to_change$old)
+  colnames(out) <- names_to_change$new
+  out <- tibble::as_tibble(out)
+  if (type == "mean") type <- "time"
+  name_with_type <- rlang::sym(paste0('.pred_', type))
+  dplyr::rename(out, !!name_with_type := .pred)
 }
